program mapcat;
{$APPTYPE CONSOLE}
uses
  Windows,
  SysUtils,
  Graphics,
  Classes,
  Math,
  jpeg;

const
  IWidth = 256;
  IHeight = 256;

function HexToInt(HexStr: String): Integer;
begin
  result := StrToInt('$' + HexStr);
end;

procedure BitmapCreateHeader(Stream: TStream; Width, Height: Cardinal);
var
  FHeader: TBitmapFileHeader;
  FInfo: TBitmapInfoHeader;
begin
  with FHeader do begin
    bfType := $4D42;
    bfReserved1 := 0;
    bfReserved2 := 0;
    bfOffBits := 14 + 40;
    bfSize := bfOffBits + Width * Height * 3;
  end;
  Stream.Write(FHeader, SizeOf(FHeader));

  with FInfo do begin
    biSize := 40;
    biWidth := Width;
    biHeight := Height;
    biPlanes := 1;
    biBitCount := 24;
    biCompression := 0;
    biSizeImage := 0;    ////
    biXPelsPerMeter := 0;
    biYPelsPerMeter := 0;
    biClrUsed := 0;
    biClrImportant := 0;
  end;
  Stream.Write(FInfo, SizeOf(FInfo));
end;

procedure BitmapSeekToData(Stream: TStream);
var
  FHeader: TBitmapFileHeader;
begin
  with Stream do begin
    Seek(0, soFromBeginning);
    Read(FHeader, SizeOf(FHeader));
    Seek(FHeader.bfOffBits, soFromBeginning);
  end;
end;

PROCEDURE ConvertBMPtoJPG (CONST BMPName: STRING; JPGName: STRING);
  VAR
    Bitmap: TBitmap;
    JPEG : TJPEGImage;
BEGIN
  Bitmap := TBitmap.Create;
  JPEG := TJPEGImage.Create;
  TRY
    Bitmap.LoadFromFile(BMPName);
    JPEG.CompressionQuality := 60;
    JPEG.Assign(Bitmap);
    JPEG.SaveToFile(JPGName);
  FINALLY
    Bitmap.Free;
    JPEG.Free
  END
END {ConvertBMPtoJPG};

var
  ImageList: TList;
  FileName: string;
  Bitmap: TBitmap = nil;
  MemoryBitmap: TMemoryStream = nil;
  Output: TFileStream;
  X, Y, J: Integer;
  XStart, XStop, YStart, YStop: Integer;
  UTM_X1, UTM_X2, UTM_Y1, UTM_Y2: Integer;
  OutFileName: string;
  FJpegImage: TJPEGImage;
  F: TextFile;
  S: string;

begin
  if ParamCount = 0 then begin
    WriteLn('MapCat -- (c) 2006 Ales Povalac');
    WriteLn('Zadej souradnice ve tvaru ????000h (napr. 7eb1000h jako "7eb1")');
    Write('X pocatek: '); ReadLn(S);
    XStart := HexToInt(S);
    Write('X konec: '); ReadLn(S);
    XStop := HexToInt(S);
    Write('Y pocatek: '); ReadLn(S);
    YStart := HexToInt(S);
    Write('Y konec: '); ReadLn(S);
    YStop := HexToInt(S);
    Write('Nazev vyst. souboru (bez .bmp, .map)? '); ReadLn(OutFileName);
  end else if ParamCount <> 5 then begin
    WriteLn('mapcat xstart xstop ystart ystop output_file_name_bez_pripony');
    Exit;
  end else begin
    XStart := HexToInt(ParamStr(1));
    XStop := HexToInt(ParamStr(2));
    YStart := HexToInt(ParamStr(3));
    YStop := HexToInt(ParamStr(4));
    OutFileName := ParamStr(5);
  end;

  ImageList := TList.Create;
  Output := TFileStream.Create(OutFileName + '.bmp', fmCreate);
  Assert(IWidth mod 4 = 0);

  for Y := YStart to YStop do begin
    Write('Nacitam ');

    { nacteni JPEGu, prevod do bitmap, ulozeni do listu }
    for X := XStart to XStop do begin
      FileName := LowerCase(Format('ophoto\16_%4x000_%4x000.jpg', [X, Y]));
      Write(FileName + ' ');

      MemoryBitmap := TMemoryStream.Create;
      FJpegImage := TJPEGImage.Create;
      Bitmap := TBitmap.Create;
      try
        Bitmap.PixelFormat := pf24bit;
        FJpegImage.LoadFromFile(FileName);
        Bitmap.Assign(FJpegImage);
        Bitmap.SaveToStream(MemoryBitmap);
        BitmapSeekToData(MemoryBitmap);
      finally
        FJpegImage.Free;
        Bitmap.Free;
      end;
      ImageList.Add(MemoryBitmap);
    end;
    WriteLn;
    WriteLn('ImageList nacten');

    if Y = YStart then begin
      WriteLn('Inicializuji Result');
      BitmapCreateHeader(Output, (XStop - XStart + 1) * IWidth, (YStop - YStart + 1) * IHeight);
      WriteLn('Result inicializovan');
    end;

    Write('Spojuji ');
    for J := IHeight - 1 downto 0 do begin
      for X := 0 to (XStop - XStart) do begin
        MemoryBitmap := TMemoryStream(ImageList.Items[X]);
        Output.CopyFrom(MemoryBitmap, IWidth*3);
      end;
      if J mod 10 = 0 then Write('.');
    end;
    WriteLn;

    WriteLn('Radek ' + IntToHex(Y, 4) + '000 pridan');

    { uvolneni listu }
    while ImageList.Count > 0 do begin
      TMemoryStream(ImageList.Items[0]).Free;
      ImageList.Delete(0);
    end;
    WriteLn('ImageList uvolnen');
  end;

  ImageList.Free;
  WriteLn('Ukladam ResultImage...');
  Output.Free;;


  WriteLn('Generuji MAP soubor...');
  AssignFile(F, OutFileName + '.map');
  ReWrite(F);
  WriteLn(F, 'OziExplorer Map Data File Version 2.2');
  WriteLn(F, OutFileName);
  WriteLn(F, OutFileName + '.jpg'); // .bmp ???
  WriteLn(F, '1 ,Map Code,');
  WriteLn(F, 'WGS 84,WGS 84,   0.0000,   0.0000,WGS 84');
  WriteLn(F, 'Reserved 1');
  WriteLn(F, 'Reserved 2');
  WriteLn(F, 'Magnetic Variation,,,E');
  WriteLn(F, 'Map Projection,(UTM) Universal Transverse Mercator,PolyCal,No,AutoCalOnly,No,BSBUseWPX,No');

  UTM_X1 := (XStart * 4096) div 32 - 3700000;
  UTM_X2 := ((XStop+1) * 4096) div 32 - 3700000;
  UTM_Y1 := (YStart * 4096) div 32 + 1300000;
  UTM_Y2 := ((YStop+1) * 4096) div 32 +1300000;

  WriteLn(F, Format('Point%.02d,xy,%5d,%5d,in, deg,    ,        ,N,    ,        ,W, grid, 33,%11d,%11d,N',
                    [1, 0, 0,
                     UTM_X1, UTM_Y2]));
  WriteLn(F, Format('Point%.02d,xy,%5d,%5d,in, deg,    ,        ,N,    ,        ,W, grid, 33,%11d,%11d,N',
                    [2, (XStop - XStart + 1) * IWidth - 1, (YStop - YStart + 1) * IHeight - 1,
                     UTM_X2, UTM_Y1]));

  for J := 3 to 30 do
    WriteLn(F, Format('Point%.02d,xy,     ,     ,in, deg,    ,        ,N,    ,        ,W, grid,   ,           ,           ,N', [J]));

  WriteLn(F, 'Projection Setup,,,,,,,,,,');
  WriteLn(F, 'Map Feature = MF ; Map Comment = MC     These follow if they exist');
  WriteLn(F, 'Track File = TF      These follow if they exist');
  WriteLn(F, 'Moving Map Parameters = MM?    These follow if they exist');
  CloseFile(F);

  WriteLn('Konvertuji BMP -> JPEG');
  ConvertBMPtoJPG(
    ExtractFilePath(ParamStr(0)) + OutFileName + '.bmp',
    ExtractFilePath(ParamStr(0)) + OutFileName + '.jpg'
  );

  WriteLn('Hotovo');
end.
